/*
 * Copyright(C) 2010 avanza Co.,Ltd. All rights reserved.
 * http://www.avnz.co.jp/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

package com.aimluck.eip.enquete;

import java.util.ArrayList;
import java.util.List;
import java.util.jar.Attributes;

import org.apache.cayenne.exp.Expression;
import org.apache.cayenne.exp.ExpressionFactory;
import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.turbine.util.RunData;
import org.apache.velocity.context.Context;

import com.aimluck.commons.field.ALStringField;
import com.aimluck.commons.utils.ALDateUtil;
import com.aimluck.eip.cayenne.om.portlet.AvzTEnquete;
import com.aimluck.eip.cayenne.om.portlet.AvzTEnqueteChoiceItem;
import com.aimluck.eip.cayenne.om.portlet.AvzTEnqueteItem;
import com.aimluck.eip.common.ALAbstractSelectData;
import com.aimluck.eip.common.ALDBErrorException;
import com.aimluck.eip.common.ALData;
import com.aimluck.eip.common.ALEipConstants;
import com.aimluck.eip.common.ALPageNotFoundException;
import com.aimluck.eip.enquete.util.EnqueteUtils;
import com.aimluck.eip.modules.actions.common.ALAction;
import com.aimluck.eip.orm.Database;
import com.aimluck.eip.orm.query.ResultList;
import com.aimluck.eip.orm.query.SelectQuery;
import com.aimluck.eip.util.ALCommonUtils;
import com.aimluck.eip.util.ALEipUtils;

/**
 * Enquete検索データを管理するクラスです。 <BR>
 * 
 */
public class EnqueteSelectData extends
    ALAbstractSelectData<AvzTEnquete, AvzTEnquete> implements ALData {

  /** logger */
  private static final JetspeedLogger logger =
    JetspeedLogFactoryService.getLogger(EnqueteSelectData.class.getName());

  /** 現在選択されているタブ */
  private String currentTab;

  /** Enquete の総数 */
  private int enqueteSum;

  /** ログインユーザーID */
  private int loginuserid;

  /** アンケート作成者ID */
  private int createrid;

  /** アンケート項目タイトル */
  private ALStringField item_title;

  /**
   * 
   * @param action
   * @param rundata
   * @param context
   * @see com.aimluck.eip.common.ALAbstractSelectData#init(com.aimluck.eip.modules.actions.common.ALAction,
   *      org.apache.turbine.util.RunData, org.apache.velocity.context.Context)
   */
  @Override
  public void init(ALAction action, RunData rundata, Context context)
      throws ALPageNotFoundException, ALDBErrorException {
    super.init(action, rundata, context);

    String sort = ALEipUtils.getTemp(rundata, context, LIST_SORT_STR);
    if (sort == null || "".equals(sort)) {
      ALEipUtils.setTemp(rundata, context, LIST_SORT_STR, ALEipUtils
        .getPortlet(rundata, context)
        .getPortletConfig()
        .getInitParameter("p2a-sort"));
      logger.debug("[EnqueteSelectData] Init Parameter. : "
        + ALEipUtils
          .getPortlet(rundata, context)
          .getPortletConfig()
          .getInitParameter("p2a-sort"));
    }
    String tabParam = rundata.getParameters().getString("tab");
    currentTab = ALEipUtils.getTemp(rundata, context, "tab");
    if (tabParam == null && currentTab == null) {
      ALEipUtils.setTemp(rundata, context, "tab", "list");
      currentTab = "list";
    } else if (tabParam != null) {
      ALEipUtils.setTemp(rundata, context, "tab", tabParam);
      currentTab = tabParam;
    }

    loginuserid = Integer.valueOf(ALEipUtils.getUserId(rundata));

  }

  /**
   * 一覧データを取得します。 <BR>
   * 
   * @param rundata
   * @param context
   * @return
   * @see com.aimluck.eip.common.ALAbstractListData#selectData(org.apache.turbine.util.RunData,
   *      org.apache.velocity.context.Context)
   */
  @Override
  public ResultList<AvzTEnquete> selectList(RunData rundata, Context context) {
    try {
      SelectQuery<AvzTEnquete> query = getSelectQuery(rundata, context);
      buildSelectQueryForListView(query);
      buildSelectQueryForListViewSort(query, rundata, context);

      ResultList<AvzTEnquete> list = query.getResultList();

      EnqueteUtils.setPublicStatus(list);
      // Enquete の総数をセットする．
      enqueteSum = list.size();

      return list;

    } catch (Exception ex) {
      logger.error("Exception", ex);
      return null;
    }
  }

  /**
   * 検索条件を設定した SelectQuery を返します。 <BR>
   * 
   * @param rundata
   * @param context
   * @return
   */
  private SelectQuery<AvzTEnquete> getSelectQuery(RunData rundata,
      Context context) {

    SelectQuery<AvzTEnquete> query = Database.query(AvzTEnquete.class);

    // 回答ユーザーとログインユーザーが一致するデータを取得
    Expression exp =
      ExpressionFactory.matchExp(AvzTEnquete.AVZ_TENQUETE_CHOICE_ITEM_PROPERTY
        + "."
        + AvzTEnqueteChoiceItem.ANSWER_ID_PROPERTY, Integer.valueOf(ALEipUtils
        .getUserId(rundata)));

    // 未回答のアンケートを取得
    Expression exp2 =
      ExpressionFactory.matchExp(
        AvzTEnquete.AVZ_TENQUETE_CHOICE_ITEM_PROPERTY
          + "."
          + AvzTEnqueteChoiceItem.RESPONSE_STATUS_PROPERTY,
        EnqueteUtils.RESPONSE_STATUS_BEFORE);

    // 「未公開」以外のアンケートを取得
    Expression exp3 =
      ExpressionFactory.noMatchExp(
        AvzTEnquete.PUBLIC_STATUS_PROPERTY,
        EnqueteUtils.PUBLIC_STATUS_BEFORE);

    // 作成ユーザーとログインユーザーが一致するデータを取得
    Expression exp4 =
      ExpressionFactory.matchExp(AvzTEnquete.CREATER_ID_PROPERTY, Integer
        .valueOf(ALEipUtils.getUserId(rundata)));

    if ("list".equals(currentTab)) {
      query.setQualifier((exp.andExp(exp2).andExp(exp3)));
    } else if ("create".equals(currentTab)) {
      query.setQualifier(exp4);
    }
    query.distinct(true);
    return buildSelectQueryForFilter(query, rundata, context);
  }

  // /**
  // * 一覧データを取得します。 <BR>
  // *
  // * @param rundata
  // * @param context
  // * @return
  // * @see
  // com.aimluck.eip.common.ALAbstractListData#selectData(org.apache.turbine.util.RunData,
  // * org.apache.velocity.context.Context)
  // */
  // @Override
  // public ResultList<AvzTEnquete> selectList(RunData rundata, Context context)
  // {
  // ResultList<AvzTEnquete> list = new ResultList<AvzTEnquete>();
  // try {
  // SelectQuery<AvzTEnquete> query = getSelectQuery(rundata, context);
  // if (query != null) {
  // buildSelectQueryForListView(query);
  // buildSelectQueryForListViewSort(query, rundata, context);
  //
  // list = query.getResultList();
  // EnqueteUtils.setPublicStatus(list);
  // }
  // // Enquete の総数をセットする．
  // enqueteSum = list.size();
  // return list;
  // } catch (Exception ex) {
  // logger.error("Exception", ex);
  // return null;
  // }
  // }
  //
  // /**
  // * 検索条件を設定した SelectQuery を返します。 <BR>
  // *
  // * @param rundata
  // * @param context
  // * @return
  // */
  // private SelectQuery<AvzTEnquete> getSelectQuery(RunData rundata,
  // Context context) {
  //
  // SelectQuery<AvzTEnqueteChoiceItem> choicequery =
  // Database.query(AvzTEnqueteChoiceItem.class);
  //
  // // 回答ユーザーとログインユーザーが一致するデータを取得
  // Expression exp00 =
  // ExpressionFactory.matchExp(
  // AvzTEnqueteChoiceItem.ANSWER_ID_PROPERTY,
  // ALEipUtils.getUserId(rundata));
  // choicequery.setQualifier(exp00);
  //
  // // 未回答のアンケートを取得
  // Expression exp01 =
  // ExpressionFactory.matchExp(
  // AvzTEnqueteChoiceItem.RESPONSE_STATUS_PROPERTY,
  // EnqueteUtils.RESPONSE_STATUS_BEFORE);
  // choicequery.andQualifier(exp01);
  //
  // ResultList<AvzTEnqueteChoiceItem> result = choicequery.getResultList();
  // List<Integer> enquete_ids = new ArrayList<Integer>();
  // for (AvzTEnqueteChoiceItem e : result) {
  // if (!enquete_ids.contains(e.getEnqueteId())) {
  // enquete_ids.add(e.getEnqueteId());
  // }
  // }
  //
  // SelectQuery<AvzTEnquete> query = Database.query(AvzTEnquete.class);
  //
  // // 回答ユーザーとログインユーザーが一致するデータを取得
  // // Expression exp =
  // // ExpressionFactory.matchExp(AvzTEnquete.AVZ_TENQUETE_CHOICE_ITEM_PROPERTY
  // // + "."
  // // + AvzTEnqueteChoiceItem.ANSWER_ID_PROPERTY, Integer.valueOf(ALEipUtils
  // // .getUserId(rundata)));
  //
  // if (enquete_ids.size() == 0) {
  // return null;
  // }
  //
  // Expression exp =
  // ExpressionFactory.inDbExp(AvzTEnquete.ENQUETE_ID_PK_COLUMN, enquete_ids);
  //
  // // 未回答のアンケートを取得
  // // Expression exp2 =
  // // ExpressionFactory.matchExp(
  // // AvzTEnquete.AVZ_TENQUETE_CHOICE_ITEM_PROPERTY
  // // + "."
  // // + AvzTEnqueteChoiceItem.RESPONSE_STATUS_PROPERTY,
  // // EnqueteUtils.RESPONSE_STATUS_BEFORE);
  //
  // // 「未公開」以外のアンケートを取得
  // Expression exp3 =
  // ExpressionFactory.noMatchExp(
  // AvzTEnquete.PUBLIC_STATUS_PROPERTY,
  // EnqueteUtils.PUBLIC_STATUS_BEFORE);
  //
  // // 作成ユーザーとログインユーザーが一致するデータを取得
  // Expression exp4 =
  // ExpressionFactory.matchExp(AvzTEnquete.CREATER_ID_PROPERTY, Integer
  // .valueOf(ALEipUtils.getUserId(rundata)));
  //
  // if ("list".equals(currentTab)) {
  // query.setQualifier((exp.andExp(exp3)));
  // } else if ("create".equals(currentTab)) {
  // query.setQualifier(exp4);
  // }
  // return buildSelectQueryForFilter(query, rundata, context);
  // }

  /**
   * ResultData に値を格納して返します。（一覧データ） <BR>
   * 
   * @param obj
   * @return
   * @see com.aimluck.eip.common.ALAbstractSelectData#getListData(java.lang.Object)
   */
  @Override
  protected Object getResultData(AvzTEnquete obj) {
    try {

      AvzTEnquete record = obj;
      EnqueteResultData rd = new EnqueteResultData();

      rd.initField();
      rd.setEnqueteId(record.getEnqueteId().intValue());
      rd.setCreaterId(record.getCreaterId().intValue());
      rd
        .setName(ALCommonUtils.compressString(record.getName(), getStrLength()));
      rd.setComment(record.getComment());
      rd.setPublicStatus(record.getPublicStatus());
      rd.setDeadlineDate(record.getDeadlineDate());
      rd.setDeadlineMethod(record.getDeadlineMethod());

      return rd;

    } catch (Exception ex) {
      logger.error("Exception", ex);
      return null;
    }
  }

  /**
   * 詳細データを取得します。 <BR>
   * 
   * @param rundata
   * @param context
   * @return
   * @see com.aimluck.eip.common.ALAbstractSelectData#selectDetail(org.apache.turbine.util.RunData,
   *      org.apache.velocity.context.Context)
   */
  @Override
  public AvzTEnquete selectDetail(RunData rundata, Context context)
      throws ALPageNotFoundException {
    try {
      AvzTEnquete enquete = EnqueteUtils.getAvzTEnquete(rundata, context);

      SelectQuery<AvzTEnqueteItem> query =
        Database.query(AvzTEnqueteItem.class);

      Expression exp =
        ExpressionFactory.matchDbExp(AvzTEnqueteItem.AVZ_TENQUETE_PROPERTY
          + "."
          + AvzTEnquete.ENQUETE_ID_PK_COLUMN, ALEipUtils.getTemp(
          rundata,
          context,
          ALEipConstants.ENTITY_ID));
      query.setQualifier(exp);
      query.orderAscending(AvzTEnqueteItem.ITEM_NUMBER_PROPERTY);

      List<AvzTEnqueteItem> list = query.getResultList();
      ArrayList<String> itemlist = new ArrayList<String>();

      for (int i = 0; i < list.size(); i++) {

        AvzTEnqueteItem e = list.get(i);
        itemlist.add(e.getItemTitle());

        item_title = new ALStringField();
        item_title.setValue(itemlist.get(0));

        for (int j = 1; j < itemlist.size(); j++) {
          item_title.setValue(item_title + " , " + itemlist.get(j));
        }

      }

      createrid = Integer.valueOf(enquete.getCreaterId());

      return enquete;

    } catch (ALPageNotFoundException pageNotFound) {
      throw pageNotFound;
    }
  }

  /**
   * ResultData に値を格納して返します。（詳細データ） <BR>
   * 
   * @param obj
   * @return
   * @see com.aimluck.eip.common.ALAbstractSelectData#getResultDataDetail(java.lang.Object)
   */
  @Override
  protected Object getResultDataDetail(AvzTEnquete obj) {
    try {
      AvzTEnquete record = obj;
      EnqueteResultData rd = new EnqueteResultData();
      rd.initField();
      rd.setEnqueteId(record.getEnqueteId().intValue());
      rd.setCreaterId(record.getCreaterId().intValue());
      rd
        .setName(ALCommonUtils.compressString(record.getName(), getStrLength()));
      rd.setUpdateDate(ALDateUtil.format(record.getUpdateDate(), "yyyy年M月d日"));
      rd.setCreateDate(ALDateUtil.format(record.getCreateDate(), "yyyy年M月d日"));
      rd.setComment(record.getComment());
      rd.setPublicStatus(record.getPublicStatus());
      rd.setDeadlineDate(record.getDeadlineDate());
      rd.setDeadlineMethod(record.getDeadlineMethod());
      ALStringField creatername = new ALStringField();
      creatername.setValue(ALEipUtils.getUserFullName(record.getCreaterId()));
      rd.setCreaterName(creatername);

      return rd;
    } catch (Exception ex) {
      logger.error("Exception", ex);
      return null;
    }
  }

  /**
   * Enquete の総数を返す． <BR>
   * 
   * @return
   */
  public int getEnqueteSum() {
    return enqueteSum;
  }

  /**
   * @return
   * @see com.aimluck.eip.common.ALAbstractSelectData#getColumnMap()
   */
  @Override
  protected Attributes getColumnMap() {
    Attributes map = new Attributes();
    map.putValue("deadline_date", AvzTEnquete.DEADLINE_DATE_PROPERTY);
    return map;
  }

  // ログインユーザーIDを取得
  public int getUserId() {
    return loginuserid;
  }

  // 現在選択されているタブを取得
  public String getCurrentTab() {
    return currentTab;
  }

  // アンケート作成者IDを取得
  public int getCreaterId() {
    return createrid;
  }

  // 項目タイトルを取得
  public ALStringField getItemTitle() {
    return item_title;
  }

}
