/*
 * Copyright(C) 2012 - 2013 orinos Co.,Ltd. All rights reserved.
 * http://www.orinos.co.jp/
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.aimluck.eip.improveope;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.cayenne.exp.ExpressionFactory;
import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.turbine.services.TurbineServices;
import org.apache.turbine.util.RunData;
import org.apache.velocity.context.Context;

import com.aimluck.commons.field.ALDateTimeField;
import com.aimluck.commons.field.ALNumberField;
import com.aimluck.commons.field.ALStringField;
import com.aimluck.eip.cayenne.om.portlet.OriTImproveopeApply;
import com.aimluck.eip.cayenne.om.portlet.OriTImproveopeApplyAction;
import com.aimluck.eip.cayenne.om.portlet.OriTImproveopeApplyActionFile;
import com.aimluck.eip.cayenne.om.portlet.OriTImproveopeApplyActionMap;
import com.aimluck.eip.cayenne.om.portlet.OriTImproveopeApplyActionReportFile;
import com.aimluck.eip.cayenne.om.security.TurbineUser;
import com.aimluck.eip.common.ALAbstractFormData;
import com.aimluck.eip.common.ALDBErrorException;
import com.aimluck.eip.common.ALEipUser;
import com.aimluck.eip.common.ALPageNotFoundException;
import com.aimluck.eip.fileupload.beans.FileuploadBean;
import com.aimluck.eip.fileupload.beans.FileuploadLiteBean;
import com.aimluck.eip.fileupload.util.FileuploadUtils;
import com.aimluck.eip.improveope.util.ImproveopeApplyUtils;
import com.aimluck.eip.modules.actions.common.ALAction;
import com.aimluck.eip.modules.actions.improveope.ImproveopeAction;
import com.aimluck.eip.modules.screens.ImproveopeReportFormScreen;
import com.aimluck.eip.orm.Database;
import com.aimluck.eip.orm.query.SelectQuery;
import com.aimluck.eip.services.accessctl.ALAccessControlConstants;
import com.aimluck.eip.services.accessctl.ALAccessControlFactoryService;
import com.aimluck.eip.services.accessctl.ALAccessControlHandler;
import com.aimluck.eip.services.eventlog.ALEventlogConstants;
import com.aimluck.eip.services.eventlog.ALEventlogFactoryService;
import com.aimluck.eip.util.ALEipUtils;

/**
 * 実施要項作成フォームデータを管理するクラスです。 <BR>
 *
 */
public class ImproveopeReportFormData extends ALAbstractFormData {

  /** logger */
  private static final JetspeedLogger logger = JetspeedLogFactoryService
    .getLogger(ImproveopeReportFormData.class.getName());

  /** 目標 */
  private ALStringField goal;

  /** 内容 */
  private ALStringField note;

  /** 実施開始日 */
  private ALDateTimeField start_date;

  /** 実施終了日 */
  private ALDateTimeField end_date;

  /** 目標に対する実績 */
  private ALStringField report1;

  /** 報告 */
  private ALStringField report2;

  /** 報告者ID */
  private ALNumberField report_user_id;

  /** 報告添付ファイルリスト */
  private List<FileuploadLiteBean> fileuploadList = null;

  /** 報告要項添付ファイルリスト */
  private List<FileuploadBean> attachmentReportFileList = null;

  /** 実施要項添付ファイルリスト */
  private List<FileuploadBean> attachmentFileList = null;

  /** 添付フォルダ名 */
  private String folderName = null;

  /** 実施対象者ユーザIDリスト */
  private ALStringField members;

  /** 実施対象者一覧 */
  private List<ALEipUser> mapList;

  /** アクションID */
  private ALStringField action_id;

  private int uid;

  private String orgId;

  private String aclPortletFeature = null;

  /** 管理者権限の有無 */
  @SuppressWarnings("unused")
  private boolean hasAclImproveopeAdmin;
  @SuppressWarnings("unused")
  private boolean hasAclImproveopeUpdate;

  /**
   *
   * @param action
   * @param rundata
   * @param context
   *
   *
   */
  @Override
  public void init(ALAction action, RunData rundata, Context context)
      throws ALPageNotFoundException, ALDBErrorException {
    super.init(action, rundata, context);

    uid = ALEipUtils.getUserId(rundata);
    orgId = Database.getDomainName();

    folderName = rundata.getParameters().getString("folderName");

    ALAccessControlFactoryService aclservice =
      (ALAccessControlFactoryService) ((TurbineServices) TurbineServices
        .getInstance()).getService(ALAccessControlFactoryService.SERVICE_NAME);
    ALAccessControlHandler aclhandler = aclservice.getAccessControlHandler();

    hasAclImproveopeAdmin =
      aclhandler.hasAuthority(
        ALEipUtils.getUserId(rundata),
        ALAccessControlConstants.POERTLET_FEATURE_IMPROVEOPE_ADMIN,
        ALAccessControlConstants.VALUE_ACL_LIST);

    hasAclImproveopeUpdate =
    	      aclhandler.hasAuthority(
    	        ALEipUtils.getUserId(rundata),
    	        ALAccessControlConstants.POERTLET_FEATURE_IMPROVEOPE_ADMIN,
    	        ALAccessControlConstants.VALUE_ACL_UPDATE);
  }

  /**
   * 各フィールドを初期化します。 <BR>
   *
   *
   */
  @Override
  public void initField() {

    // 目標
	goal = new ALStringField();
	goal.setFieldName("目標");
	goal.setTrim(false);

    // 内容
    note = new ALStringField();
    note.setFieldName("実施内容");
    note.setTrim(false);

    // 実施開始日
    start_date = new ALDateTimeField("yyyy-MM-dd-HH-mm");
    start_date.setValue(new Date());

    // 実施終了日
    end_date = new ALDateTimeField("yyyy-MM-dd-HH-mm");
    end_date.setValue(new Date());

    // 目標に対する実績
    report1 = new ALStringField();
    report1.setFieldName("目標に対する実績");
    report1.setTrim(false);

    // 報告
    report2 = new ALStringField();
    report2.setFieldName("報告内容");
    report2.setTrim(false);

    // 報告者
    report_user_id = new ALNumberField();
    report_user_id.setFieldName("報告者");

    // 実施対象者リスト
    members = new ALStringField();
    members.setFieldName("実施対象者");
    members.setTrim(true);
    // 実施対象者リスト
    mapList = new ArrayList<ALEipUser>();

    fileuploadList = new ArrayList<FileuploadLiteBean>();
    attachmentFileList = new ArrayList<FileuploadBean>();
    attachmentReportFileList = new ArrayList<FileuploadBean>();

    action_id = new ALStringField();

    aclPortletFeature =
      ALAccessControlConstants.POERTLET_FEATURE_IMPROVEOPE_SELF;
  }

  /**
   * 申請の各フィールドに対する制約条件を設定します。 <BR>
   *
   *
   */
  @Override
  protected void setValidator() {
    // 実績必須項目
	report1.setNotNull(true);
    // 実績の文字数制限
	report1.limitMaxLength(10000);
    // 内容必須項目
	report2.setNotNull(true);
    // 内容の文字数制限
	report2.limitMaxLength(10000);
  }

  /**
   * トピックのフォームに入力されたデータの妥当性検証を行います。 <BR>
   *
   * @param msgList
   * @return TRUE 成功 FALSE 失敗
   *
   */
  @Override
  protected boolean validate(List<String> msgList) {
    // 実績
	report1.validate(msgList);
    // 内容
	report2.validate(msgList);

    return (msgList.size() == 0);

  }

  /**
   * トピックをデータベースから読み出します。 <BR>
   *
   * @param rundata
   * @param context
   * @param msgList
   * @return TRUE 成功 FALSE 失敗
   */
  @Override
  protected boolean loadFormData(RunData rundata, Context context,
      List<String> msgList) {
    try {

        // オブジェクトモデルを取得
        OriTImproveopeApplyAction action =
          ImproveopeApplyUtils.getOriTImproveopeApplyAction(rundata, context);
        if (action == null) {
          return false;
        }

        // ID
        action_id.setValue(action.getActionId().toString());

        // 目標
        goal.setValue(action.getGoal());

        // 内容
        note.setValue(action.getNote());

        // 目標に対する実績
        report1.setValue(action.getReport1());

        // 報告内容
        report2.setValue(action.getReport2());

        // 報告者
        if (action.getReportUserId() != null) {
        	report_user_id.setValue(action.getReportUserId());
        }

        Date now = Calendar.getInstance().getTime();// 現在時刻取得
        // 実施開始日を取得
        if (action.getStartDate() != null) {
      	    start_date.setValue(action.getStartDate());
        } else {
        	start_date.setValue(now);
        }

        // 実施終了日を取得
        if (action.getEndDate() != null) {
      	    end_date.setValue(action.getEndDate());
        } else {
        	end_date.setValue(now);
        }

        // 報告ファイル
        SelectQuery<OriTImproveopeApplyActionReportFile> queryReport =
          Database.query(OriTImproveopeApplyActionReportFile.class);
        queryReport.andQualifier(ExpressionFactory.matchDbExp(
        		OriTImproveopeApplyActionReportFile.ORI_TIMPROVEOPE_APPLY_ACTION_PROPERTY,
        		action.getActionId()));
        List<OriTImproveopeApplyActionReportFile> improveopeReportFileList = queryReport.fetchList();
        for (OriTImproveopeApplyActionReportFile file : improveopeReportFileList) {
          FileuploadLiteBean fbean = new FileuploadLiteBean();
          fbean.initField();
          fbean.setFileId(file.getFileId());
          fbean.setFileName(file.getFileName());
          fileuploadList.add(fbean);
        }
        if (improveopeReportFileList != null && improveopeReportFileList.size() > 0) {
            FileuploadBean filebean = null;
            OriTImproveopeApplyActionReportFile file = null;
            int size = improveopeReportFileList.size();
            for (int i = 0; i < size; i++) {
              file = improveopeReportFileList.get(i);
              String realname = file.getFileName();
              javax.activation.DataHandler hData =
                new javax.activation.DataHandler(
                  new javax.activation.FileDataSource(realname));

              filebean = new FileuploadBean();
              filebean.setFileId(file.getFileId().intValue());
              filebean.setFileName(realname);
              if (hData != null) {
                filebean.setContentType(hData.getContentType());
              }
              filebean.setIsImage(FileuploadUtils.isImage(realname));
              attachmentReportFileList.add(filebean);
            }
          }

        // 実施要項ファイル
        SelectQuery<OriTImproveopeApplyActionFile> query =
                Database.query(OriTImproveopeApplyActionFile.class);
              query.andQualifier(ExpressionFactory.matchDbExp(
              		OriTImproveopeApplyActionFile.ORI_TIMPROVEOPE_APPLY_ACTION_PROPERTY,
              		action.getActionId()));
              List<OriTImproveopeApplyActionFile> improveopeFileList = query.fetchList();
        if (improveopeFileList != null && improveopeFileList.size() > 0) {
          FileuploadBean filebean = null;
          OriTImproveopeApplyActionFile file = null;
          int size = improveopeFileList.size();
          for (int i = 0; i < size; i++) {
            file = improveopeFileList.get(i);
            String realname = file.getFileName();
            javax.activation.DataHandler hData =
              new javax.activation.DataHandler(
                new javax.activation.FileDataSource(realname));

            filebean = new FileuploadBean();
            filebean.setFileId(file.getFileId().intValue());
            filebean.setFileName(realname);
            if (hData != null) {
              filebean.setContentType(hData.getContentType());
            }
            filebean.setIsImage(FileuploadUtils.isImage(realname));
            attachmentFileList.add(filebean);
          }
        }

        // 実施対象者
        List<OriTImproveopeApplyActionMap> maps = ImproveopeApplyUtils.getOriTImproveopeApplyActionMap(action);
        OriTImproveopeApplyActionMap map = null;
        int size = maps.size();
        for (int i = 0; i < size; i++) {
          map = maps.get(i);
          int user_id = map.getUserId().intValue();
          mapList.add(ALEipUtils.getALEipUser(user_id));
        }

    } catch (Exception ex) {
      logger.error("Exception", ex);
      return false;
    }
    return true;
  }


  /**
   * データベースに格納されているトピックを更新します。 <BR>
   *
   * @param rundata
   * @param context
   * @param msgList
   * @return TRUE 成功 FALSE 失敗
   */
  @Override
  protected boolean updateFormData(RunData rundata, Context context,
      List<String> msgList) {
    try {

      // 親オブジェクト
      OriTImproveopeApply apply =
        ImproveopeApplyUtils.getOriTImproveopeApply(rundata, context);
      // オブジェクトモデル
      OriTImproveopeApplyAction action =
        ImproveopeApplyUtils.getOriTImproveopeApplyAction(rundata, context);
      // 目標に対する実績
      action.setReport1(report1.getValue());
      // 報告内容
      action.setReport2(report2.getValue());
      // 報告者
      action.setReportUserId(Integer.valueOf(uid));
      // 更新者
      action.setUpdateUserId(Integer.valueOf(uid));
      // 更新日
      action.setUpdateDate(Calendar.getInstance().getTime());
      // ファイルをデータベースに登録する．
      if (!ImproveopeApplyUtils.insertFileDataDelegateReport(
        rundata,
        context,
        action,
        fileuploadList,
        folderName,
        msgList)) {
        return false;
      }

      Database.commit();

      // イベントログに保存
      ALEventlogFactoryService.getInstance().getEventlogHandler().log(
    	action.getActionId(),
        ALEventlogConstants.PORTLET_TYPE_IMPROVEOPE,
        apply.getApplyName());


      /* 本部権限保持者へアクティビティー */
      List<String> ActivityList = ImproveopeApplyUtils.getAdminMemberList();
      TurbineUser tuser = null;
      List<OriTImproveopeApplyActionMap> maps = ImproveopeApplyUtils.getOriTImproveopeApplyActionMap(action);
      for (OriTImproveopeApplyActionMap user : maps) {
        try {
          tuser = ALEipUtils.getTurbineUser(Integer.valueOf(user.getUserId().toString()));
          if (ActivityList.indexOf(tuser.getLoginName()) == -1) {
            ActivityList.add(tuser.getLoginName());
          }
        } catch (ALDBErrorException e) {}
      }
      ALEipUser user = ALEipUtils.getALEipUser(uid);
      ImproveopeApplyUtils.createReportActivity(
    	apply,
        user.getName().getValue(),
        ActivityList,
        true);


    } catch (Exception ex) {
      Database.rollback();
      logger.error("Exception", ex);
      return false;
    }
    return true;
  }

  /**
   * 業務改善申請詳細表示ページにフォームを表示します。
   *
   * @param action
   * @param rundata
   * @param context
   * @return TRUE 成功 FALSE 失敗
   */
  @Override
  public boolean doViewForm(ALAction action, RunData rundata, Context context) {
    try {
      init(action, rundata, context);

	    // オブジェクトモデルを取得
	    OriTImproveopeApplyAction check =
	      ImproveopeApplyUtils.getOriTImproveopeApplyAction(rundata, context);
	    if (check != null) {
		    // 目標
		    goal.setValue(check.getGoal());
		    // 内容
		    note.setValue(check.getNote());
		    // ID
		    action_id.setValue(check.getActionId().toString());
	        // 目標に対する実績
	        report1.setValue(check.getReport1());
	        // 報告内容
	        report2.setValue(check.getReport2());
	        // 報告者
	        if (check.getReportUserId() != null) {
	        	report_user_id.setValue(check.getReportUserId());
	        }
	        // 実施開始日を取得
      	    start_date.setValue(check.getStartDate());
	        // 実施終了日を取得
      	    end_date.setValue(check.getEndDate());
            // 報告ファイル
            SelectQuery<OriTImproveopeApplyActionReportFile> queryReport =
              Database.query(OriTImproveopeApplyActionReportFile.class);
            queryReport.andQualifier(ExpressionFactory.matchDbExp(
            		OriTImproveopeApplyActionReportFile.ORI_TIMPROVEOPE_APPLY_ACTION_PROPERTY,
            		check.getActionId()));
            List<OriTImproveopeApplyActionReportFile> improveopeReportFileList = queryReport.fetchList();
            for (OriTImproveopeApplyActionReportFile file : improveopeReportFileList) {
              FileuploadLiteBean fbean = new FileuploadLiteBean();
              fbean.initField();
              fbean.setFileId(file.getFileId());
              fbean.setFileName(file.getFileName());
              fileuploadList.add(fbean);
            }
            if (improveopeReportFileList != null && improveopeReportFileList.size() > 0) {
                FileuploadBean filebean = null;
                OriTImproveopeApplyActionReportFile file = null;
                int size = improveopeReportFileList.size();
                for (int i = 0; i < size; i++) {
                  file = improveopeReportFileList.get(i);
                  String realname = file.getFileName();
                  javax.activation.DataHandler hData =
                    new javax.activation.DataHandler(
                      new javax.activation.FileDataSource(realname));

                  filebean = new FileuploadBean();
                  filebean.setFileId(file.getFileId().intValue());
                  filebean.setFileName(realname);
                  if (hData != null) {
                    filebean.setContentType(hData.getContentType());
                  }
                  filebean.setIsImage(FileuploadUtils.isImage(realname));
                  attachmentReportFileList.add(filebean);
                }
              }
	        // ファイル
	        SelectQuery<OriTImproveopeApplyActionFile> query =
	  	          Database.query(OriTImproveopeApplyActionFile.class);
	  	        query.andQualifier(ExpressionFactory.matchDbExp(
	  	        		OriTImproveopeApplyActionFile.ORI_TIMPROVEOPE_APPLY_ACTION_PROPERTY,
	  	        		check.getActionId()));
	  	        List<OriTImproveopeApplyActionFile> improveopeFileList = query.fetchList();
	        if (improveopeFileList != null && improveopeFileList.size() > 0) {
	          FileuploadBean filebean = null;
	          OriTImproveopeApplyActionFile file = null;
	          int size = improveopeFileList.size();
	          for (int i = 0; i < size; i++) {
	            file = improveopeFileList.get(i);
	            String realname = file.getFileName();
	            javax.activation.DataHandler hData =
	              new javax.activation.DataHandler(
	                new javax.activation.FileDataSource(realname));

	            filebean = new FileuploadBean();
	            filebean.setFileId(file.getFileId().intValue());
	            filebean.setFileName(realname);
	            if (hData != null) {
	              filebean.setContentType(hData.getContentType());
	            }
	            filebean.setIsImage(FileuploadUtils.isImage(realname));
	            attachmentFileList.add(filebean);
	          }
	        }

	        // 実施対象者
	        List<OriTImproveopeApplyActionMap> maps = ImproveopeApplyUtils.getOriTImproveopeApplyActionMap(check);
	        OriTImproveopeApplyActionMap map = null;
	        int size = maps.size();
	        for (int i = 0; i < size; i++) {
	          map = maps.get(i);
	          int user_id = map.getUserId().intValue();
	          mapList.add(ALEipUtils.getALEipUser(user_id));
	        }
	    }


      action.setMode("reply");
      List<String> msgList = new ArrayList<String>();
      boolean res = setFormData(rundata, context, msgList);
      if (action instanceof ImproveopeReportFormScreen) {
    	  ImproveopeReportFormScreen improveopeAction =
          (ImproveopeReportFormScreen) action;
        improveopeAction.setResultDataOnReportForm(this);
        improveopeAction.addErrorMessagesOnReportForm(msgList);
        improveopeAction.putDataOnReportForm(rundata, context);
      } else {
        ImproveopeAction improveopeAction = (ImproveopeAction) action;
        improveopeAction.setResultDataOnReportForm(this);
        improveopeAction.addErrorMessagesOnReportForm(msgList);
        improveopeAction.putDataOnReportForm(rundata, context);
      }
      return res;
      // } catch (ALPermissionException e) {
      // ALEipUtils.redirectPermissionError(rundata);
      // return false;
    } catch (ALPageNotFoundException e) {
      ALEipUtils.redirectPageNotFound(rundata);
      return false;
    } catch (ALDBErrorException e) {
      ALEipUtils.redirectDBError(rundata);
      return false;
    }
  }

  /**
   *
   * @param rundata
   * @param context
   * @param msgList
   * @return
   * @throws ALPageNotFoundException
   * @throws ALDBErrorException
   */
  @Override
  protected boolean setFormData(RunData rundata, Context context,
      List<String> msgList) throws ALPageNotFoundException, ALDBErrorException {

    boolean res = super.setFormData(rundata, context, msgList);

    try {

    	if (fileuploadList.size() == 0) {
	        fileuploadList = ImproveopeApplyUtils.getActionFileuploadList(rundata);
    	}

      } catch (Exception ex) {
        logger.error("Exception", ex);
      }

    return res;
  }


  /**
   * 目標を取得します。 <BR>
   *
   * @return
   */
  public String getGoal() {
    return ALEipUtils.getMessageList(goal.toString());
  }

  /**
   * メモを取得します。 <BR>
   *
   * @return
   */
  public String getNote() {
    return ALEipUtils.getMessageList(note.toString());
  }

  /**
   * IDを取得します。 <BR>
   *
   * @return
   */
  public ALStringField getId() {
    return action_id;
  }

  /**
   * 目標に対する実績を取得します。 <BR>
   *
   * @return
   */
  public String getReport1View() {
    return ALEipUtils.getMessageList(report1.toString());
  }
  /**
   * 目標に対する実績を取得します。 <BR>
   *
   * @return
   */
  public String getReport1() {
    return report1.toString();
  }

  /**
   * 報告内容を取得します。 <BR>
   *
   * @return
   */
  public String getReport2View() {
    return ALEipUtils.getMessageList(report2.toString());
  }

  /**
   * 報告内容を取得します。 <BR>
   *
   * @return
   */
  public String getReport2() {
    return report2.toString();
  }

  /**
   * 報告者を取得します。 <BR>
   *
   * @return
   */
  public String getReportUserName() {
	  if (report_user_id.toString() != null && !"".equals(report_user_id.toString())) {
	    return ALEipUtils
	            .getUserFullName(Integer.valueOf(report_user_id.toString()));
	  }else{
		  return "";
	  }
  }

  /**
   * 実施開始日を取得します。 <BR>
   *
   * @return
   */
  public ALDateTimeField getStartDate() {
    return start_date;
  }

  /**
   * 実施終了日を取得します。 <BR>
   *
   * @return
   */
  public ALDateTimeField getEndDate() {
    return end_date;
  }

  /**
   * アクセス権限チェック用メソッド。 アクセス権限の機能名を返します。
   *
   * @return
   */
  @Override
  public String getAclPortletFeature() {
    return aclPortletFeature;
  }

  public void setAclPortletFeature(String aclPortletFeature) {
    this.aclPortletFeature = aclPortletFeature;
  }

  /**
   * 実施要項添付ファイルを取得
   * @return
   */
  public List<FileuploadBean> getAttachmentFileList() {
	    return attachmentFileList;
  }

  /**
   * 報告要項添付ファイルを取得
   * @return
   */
  public List<FileuploadBean> getAttachmentReportFileList() {
	    return attachmentReportFileList;
  }


@Override
protected boolean deleteFormData(RunData rundata, Context context,
		List<String> msgList) throws ALPageNotFoundException,
		ALDBErrorException {
	// TODO 自動生成されたメソッド・スタブ
	return false;
}

/**
 * 他ユーザのトピックを編集する権限があるかどうかを返します。
 *
 * @return
 */
public boolean hasAclUpdateApply() {
  return hasAclImproveopeUpdate;
}

public List<FileuploadLiteBean> getAttachmentFileNameList() {
    return fileuploadList;
  }

  public String getFolderName() {
    return folderName;
  }

  /**
   * グループメンバーを取得します。 <BR>
   *
   * @return
   */
  public List<ALEipUser> getMapList() {
    return mapList;
  }

  /**
   * グループメンバーを格納します。 <BR>
   *
   * @param str
   * @return
   */
  public void setMapList(ArrayList<ALEipUser> list) {
    mapList = list;
  }

@Override
protected boolean insertFormData(RunData rundata, Context context,
		List<String> msgList) throws ALPageNotFoundException,
		ALDBErrorException {
	// TODO 自動生成されたメソッド・スタブ
	return false;
}

}
